//
//
//  fibonacci_i.cc
//
//      AUTHOR: Russell Lowke   Mon Feb 16th 2004
//
// Description: Calculates the Fibonacci sequence using iteration
//
//

#include <iostream>
#include <ctime>
#include <vector>

using namespace std;

typedef vector<int>    IntList;

enum {OVERFLOW_AT2_16 = 65536};
enum {OVERFLOW_AT2_32 = 4294967296};	// not used, but nice to know

//
//  iterative call that calculates a fibonacci number
//
// I have included  % OVERFLOW_AT2_16 in arithmetic, as specified,
// even though my machine does not overflow using ints at 2^16
//
unsigned int fib_i (int n) {

    IntList f(n + 1);			// +1 as [0] included
    
    f[0] = 0;
    f[1] = 1;
    
	for (int i = 2; i <= n; ++i)
        f[i] = ( f[i - 1]  + f[i - 2] ) % OVERFLOW_AT2_16;
    
    return f[n];		// OVERFLOW_AT2_16 avoids overflow
}

//
// Calculates the number of fibonacci numbers calculated in [seconds] seconds
//
int nFibsInTime(int seconds) {

    int n = 0;
	clock_t endwait =  clock() + seconds * CLK_TCK;
    while (clock() < endwait) {
        ++n;
        fib_i(n);
    }
    
    return n;

}

//
// Calculates to the [fibNum] fibonacci number [nTimes] times
//  returning the duration taken
//
clock_t timedFib(int fibNum, int nTimes) {
    clock_t startTime = clock();
    
    for (int i; i < nTimes; i++ )
        fib_i(fibNum);
    clock_t endTime = clock();
    
    return (endTime - startTime);
}

int main() {

    clock_t duration = timedFib(24, 10000);
    cout << "It took " << duration << " 1/" << CLK_TCK <<
        "ths of a second for fib_i to compute the 24th fibonacci number 10000 times" << endl;
    
    cout << endl;

    int n = nFibsInTime(60);
	cout << "After 1 minute fib_i computed " << n << " fibonacci numbers." << endl;

    return 0;
}