//
//
//  fibonacci_r.cc
//
//      AUTHOR: Russell Lowke   Mon Feb 16th 2004
//
// Description: Calculates the Fibonacci sequence recursively
//
//

#include <iostream>
#include <ctime>

using namespace std;

enum {OVERFLOW_AT2_16 = 65536};
enum {OVERFLOW_AT2_32 = 4294967296};	// not used, but nice to know


//
//  recursive call that calculates the fibonacci sequence
//
// I have included  % OVERFLOW_AT2_16 in arithmetic, as specified,
// even though my machine does not overflow using ints at 2^16
//
unsigned int fib_r (int n) {
    
    if (n <= 0)					// base case 0       = 0
		return 0;
	else if (n <= 2)			// base case 1 or 2  = 1
		return 1;
	else
		return ( (fib_r(n - 1) + fib_r(n - 2)) % OVERFLOW_AT2_16 );	// recursion
}

//
// Calculates the number of fibonacci numbers calculated in [seconds] seconds
//
int nFibsInTime(int seconds) {

    int n = 0;
	clock_t endwait =  clock() + seconds * CLK_TCK;
    while (clock() < endwait) {
        ++n;
        fib_r(n);
    }
    
    return n;

}

//
// Calculates to the [fibNum] fibonacci number [nTimes] times
//  returning the duration taken
//
clock_t timedFib(int fibNum, int nTimes) {
    clock_t startTime = clock();
    
    for (int i; i < nTimes; i++ )
        fib_r(fibNum);
    clock_t endTime = clock();
    
    return (endTime - startTime);
}

int main() {

    clock_t duration = timedFib(24, 10000);
    cout << "It took " << duration << " 1/" << CLK_TCK <<
        "ths of a second for fib_r to compute the 24th fibonacci number 10000 times" << endl;
    
    cout << endl;
    
    int n = nFibsInTime(60);
	cout << "After 1 minute fib_r computed " << n << " fibonacci numbers." << endl;
    
    
    return 0;
}
    