//
//
//  Document.cpp
//
//      AUTHOR: Russell Lowke
//
//		  Date: 3/26/2004
//

#include "Document.h"
#include <cmath>
using namespace std;
    
void Document::printDocument() {

    cout << "Optimal division for " << _filename << endl << endl;
    for (int i = 0; i < _text.size() ; ++i) {
        if (_goodBreaks.front() == i) {
            _goodBreaks.erase(_goodBreaks.begin());
            cout<<endl;
        }
        cout<< _text[i]<<" ";
    }
}


Document::Document (string filename, int lineLength) {

    cout << "Creating document from " + filename << endl;
    _filename   = filename;
    _lineLength = lineLength;
    _error      = false;
    
    fstream inputFile(filename.c_str(), ios::in);
    
    if (!inputFile) {
        cerr << "ERR: File not found" << endl;
        _error = true;
    } else {
        string word;
        while(inputFile>>word) {
        _text.push_back(word);
    }
        _knownMinPenalty = new int[_text.size()];
        _breaks = new IntList[_text.size()];
        
        for (int i = 0; i < _text.size(); ++i) {
            // array holds the minimum penalty when the document is flowed from that 
            // point on a value of -1 means that the value is currently unknown
            _knownMinPenalty[i]= UNKNOWN;
        }
    }
}


//
//    method takes an int: the starting word, and returns an int, the total penalty 
//    when everything's been optimally broken up beyond that point.
//
int Document::calculatePenalty(int startIndex) {

    // if we already know how to divide from this point on, return that knowledge
    if (_knownMinPenalty[startIndex] != UNKNOWN)
        return _knownMinPenalty[startIndex];

    // first calculate the maximum number of words to have on the line: we work back
    //   from there
    int numWords = 0;
    int currentLength = 0;

    for (int i = startIndex; i < _text.size() ; ++i) {
    
        // currentlength + " " + word > max allowed length means we must put
        // this word on the next line.  if not, just add it and move on.
        if (!(( currentLength + 1 + _text[i].length() ) > _lineLength)) {
        
            // on first word, don't add a space
            if (currentLength != 0) 
                currentLength += 1;
                
            currentLength += _text[i].length();
            ++numWords;
            
        } else
            break;
    }
    
    // check if we're on the last line.  If so, the penalty's ZERO
    if (startIndex + numWords == _text.size()) {
        for (int i = startIndex; i < _text.size(); ++i)
            _knownMinPenalty[i] = 0;

        return 0;
        
    } else {
    
        int penalty    = UNKNOWN;
        int newPenalty = UNKNOWN;
        int goodIndex  = _text.size() - 1;

        int trailingSpaces = _lineLength - currentLength;
        
        
                
        // 
        // Now the clincher: total penalty = penalty of this line + 
        //                                   penalty of rest of document
        // Note:  the clause (trailingSpaces + _text[startIndex + (i - 1)].length()) + 1)
        // exists so that the value for the trailingSpaces updates as we successively
        // remove words from the end of the line and calculate the penalty for the rest. 
        // _text[startIndex + (i - 1).length is the length of the word on the end of the 
        // line, and the +1 is, of course, for the space that was between it and the 
        // previous one
        //
        for (int i = numWords; i >= 1; --i) {
            if (i != numWords)
                trailingSpaces += 1 + _text[startIndex + (i)].length();

            newPenalty = (int) pow((double) (trailingSpaces), 3) +
                calculatePenalty(startIndex+i);
                
            if ((newPenalty < penalty) || (penalty == UNKNOWN)) {
                penalty = newPenalty;
                goodIndex = startIndex + i;
            }
        }

        // we're done, so store the minimum penalty and the knowledge of how to break
        // up the rest of the document
        _knownMinPenalty[startIndex] = penalty;
        _breaks[startIndex].push_back(goodIndex);
        for (IntList::iterator i = _breaks[goodIndex].begin();
                              i != _breaks[goodIndex].end(); ++i)
            _breaks[startIndex].push_back(*i);
            
        return penalty;
    }
}

void Document::flowDocument() {

    cout << "About to calculate penalty with " << _text.size() << 
            " words and a line length of " << _lineLength << endl;
    _documentPenalty = calculatePenalty(0);
    cout << "Penalty of document is: " << _documentPenalty << endl;
    cout << "breaks before words numbers: ";
    for (IntList::iterator i = _breaks[0].begin(); i != _breaks[0].end(); ++i)
        cout << " " << *i;
        
    _goodBreaks = _breaks[0];
    cout << endl << endl;
}