//
//
//  dynamic.cpp
//
//      AUTHOR: Russell Lowke
//
//		  Date: 3/24/2004
//

#include <cstring>
#include <iostream>
#include <ctime>
#include <vector>
#include <cmath>

#include <string>
#include <fstream>
using std::ifstream;

using namespace std;

typedef struct { int brkAfterWrd;
                 int spaces;
                 int penalty;
                 int totPenalty; } BreakDef;
typedef vector<int>		IntList;
typedef vector<string>	StringList;
typedef vector<BreakDef> BreakList;
typedef vector<BreakList> BreakLstList;

class Parse {
	  
	public:
        
	  // methods
	  Parse();											// constructor
      ~Parse();											// destructor
      Parse(int m, char *iFilename);					// constructor
      void			doParse();							//
      void			printBreaks(BreakList breaksList);
      BreakList		createBreaks();
      BreakList 	reflow(BreakList breaksList, int atIndex);
	  
	private:
 
	  int			_m;									// maximum line length
      char *		_iFilename;							// name of input file
	  StringList	_WordsList;							// list of words in .txt file
      int			_WordCount;							// number of words in _WordsList
      
      BreakList		shiftEndWord(BreakList breaksList, int atIndex);
	  BreakList		createBreaks(BreakList breaksList);
};

//
//  constructor for Parse class
//
Parse :: Parse() {}
Parse :: ~Parse() {}
Parse :: Parse(int m, char *iFilename) {			// constructor
    _m 			= m;
    _iFilename  = iFilename;
}


void Parse :: doParse() {

    ifstream inFile(_iFilename, ios::in);
    
    if(! inFile) {
        cerr << "File could not be opened" << endl;
        exit(1);
    }
    
    // generate vector of words
    string word;
	while (inFile >> word )
        _WordsList.push_back(word);
    
    _WordCount = _WordsList.size();
    
}

BreakList Parse :: createBreaks() {
    BreakList breaksList; 	// empty
    return createBreaks(breaksList);
}

BreakList Parse :: createBreaks(BreakList breaksList) {

    string   word;
    BreakDef mBreak;
    int spaces;
    int penalty;
    
    // find word to start calculating breaks from
	int atWord     = 0;
    int totPenalty = 0;
    if (breaksList.size() > 0) {
        atWord     = breaksList.back().brkAfterWrd + 1;
        totPenalty = breaksList.back().totPenalty;
	}
    int lineLength = 0;

    for (; atWord < _WordCount; ++atWord) {
    
        word = _WordsList[atWord];						// get the next word
        
        if (lineLength + word.length() + 1 > _m) {		// will this word blow m?
        
            spaces	 = _m - lineLength;					// find # spaces at end of line
            penalty  = (int) pow((double) spaces, 3);	// find penalty
            totPenalty += penalty;						// add to totPenalty
            
            // declare mBreak
            mBreak.brkAfterWrd = atWord - 1;			// set break after prev. word
            mBreak.spaces      = spaces;
            mBreak.penalty	   = penalty;
            mBreak.totPenalty  = totPenalty;
            breaksList.push_back(mBreak);				
            
            lineLength = 0;								// reset line
        } 
        
        if (lineLength != 0)							// if not at start of new line
            lineLength += 1;							// add a space
        
        lineLength += word.length();					// add word to lineLength
    }
    
    return breaksList;
}


BreakList Parse :: shiftEndWord(BreakList breaksList, int atIndex) {
        
    int brkListSize = breaksList.size();
    for (int i = atIndex + 1; i < brkListSize; ++i)
        breaksList.pop_back();							 // delete all items after atIndex
    
    // modify .brkAfterWrd at atIndex by -1 [move this word to next line]
    string word = _WordsList[breaksList.back().brkAfterWrd];	// find word used to break at
    int oSpaces     = breaksList.back().spaces;
    int oPenalty    = breaksList.back().penalty;
    int nSpaces     = oSpaces + word.size() + 1;				// +1 includes space before word
    int nPenalty    = (int) pow( (double) nSpaces, 3);
    int nTotPenalty = breaksList.back().totPenalty + (nPenalty - oPenalty);
    
    // shift the word onto the next line by correcting struct values for this line
    breaksList.back().brkAfterWrd = breaksList.back().brkAfterWrd - 1;
    breaksList.back().spaces      = nSpaces;
    breaksList.back().penalty     = nPenalty;
    breaksList.back().totPenalty  = nTotPenalty;
                 
    // createBreaks for remaining.
    breaksList = createBreaks(breaksList);
    
    return breaksList;
}

BreakList Parse :: reflow(BreakList breaksList, int atIndex) {
        
    BreakList nbreaksList = shiftEndWord(breaksList, atIndex);
    
    if (atIndex + 1 == nbreaksList.size()) {
        // skip, at last line
    } else {
        // recurse both.
        nbreaksList = reflow(nbreaksList, atIndex + 1);
        breaksList  = reflow(breaksList, atIndex + 1);
    }
        
	// return BreakList with smallest total penalty
    if (nbreaksList.back().totPenalty < breaksList.back().totPenalty)
        return nbreaksList;
    else
        return breaksList;
}

void Parse :: printBreaks(BreakList breaksList) {

    // print limits
	cout << "+";
    for (int i = 1; i < _m-1; ++i)
        cout << " ";
    cout << "+" << endl;
    
	int breakIndex = 0;
    int breakCount = breaksList.size();
    for(int i = 0; i < _WordCount; ++i) {
    
        cout << _WordsList[i] << " ";
        
        if (breakIndex < breakCount && 
            breaksList[breakIndex].brkAfterWrd == i) {	// test for line break
            
            int spaces = breaksList[breakIndex].spaces - 1;
            for(int i = 0; i < spaces; ++i)
                cout << " ";
            // cout << "\tb: " << breaksList[breakIndex].brkAfterWrd;
            cout << "\tsp: " << breaksList[breakIndex].spaces;
            cout << "\tp: " << breaksList[breakIndex].penalty;
            // cout << "\t tp: " << breaksList[breakIndex].totPenalty;
            cout << endl;								// end this line
            
            ++breakIndex;								// inc for next break pt.
        }
    }
	cout << endl;										// clean up last line

    // print limits
	cout << "+";
    for (int i = 1; i < _m-1; ++i)
        cout << " ";
    cout << "+" << endl;
    
	cout << "Total penalty = " << breaksList.back().totPenalty << endl;
}

int main(int ac, char *av[]) {
	
	// randmst 0 numpoints numtrials dimension
	
	// defaults
	int m = 40;						// maximum line length.
	char* iFilename = "Buffy.txt";
    
    if (ac == 3) {
		m         = atoi(av[1]);
        iFilename = av[2];
        // how do I convert av[2]  to a string ? should I bother?
	}

	cout << "maximum line length [m] = " << m << endl;
    cout << " input file [iFilename] = \"" << iFilename << "\"" << endl << endl;
        
    Parse obj = Parse(m, iFilename);
    obj.doParse();
    BreakList mBreaksList = obj.createBreaks();
    BreakList nBreaksList = obj.reflow(mBreaksList, 0);
	obj.printBreaks(mBreaksList);
    obj.printBreaks(nBreaksList);
    
    return 0;
}