//
//
//  assignment1.cc
//
//      AUTHOR: Russell Lowke   Mon March 11th 2004
//
// Description: 
//
//

//#include <stdio.h>
//#include <cmath>


#include "edge.hpp"
#include "graph.hpp"
#include "vertex.hpp"

#include <algorithm>
#include <iostream>
#include <ctime>
#include <vector>

using namespace std;

// return the name of the set containing the element x
Vertex* find(Vertex* mX) {
	
	// if x ≠ p(x) then
    //	p(x) := FIND(p(x))
    //	return (p(x))
	if (mX != mX->getParent())
		mX->setMySet(find(mX->getParent()));		// path compression

	return mX->getParent();
}

Vertex* link(Vertex* mX, Vertex* mY) {

    // if rank(x) > rank(y) 
	if (mX->getRank() > mY->getRank()) {
		// then switch x and y
		Vertex* mTemp = mY;
		mY = mX;
		mX = mTemp;
	}
	
    // if rank(x) = rank(y) then rank(y) := rank(y) + 1
	if (mX->getRank() == mY->getRank())
		mY->setRank(mY->getRank() + 1);
	
    // p(x) := y
	mX->setMySet(mY);
	
    // return(y)
	return mY;
}


// replace two sets containing x and y by their union
void setsUnion(Vertex* mX, Vertex* mY) {
	//     LINK(FIND(x), FIND(y))
	link(find(mX), find(mY));
}


// Function Kruskal(graph G(V,E))
//    set X
//	  X = {}
//	  E:= sort E by weight
//	  for u is an element of V
//	      MAKESET(u)
//	  rof
//	  for (u, v) is an element of E (in increasing order) do
//	      if FIND(u) ≠ FIND(v) do
//	          X = X union {(u,v)}
//	          UNION(u,v)
//    rof
//    return(X)
// end Kruskal
//
EdgeList* kruskal(Graph* mGraph) {
    
    //    set X
    //	  X = {}
	EdgeList* tree = new EdgeList;

    //	  E:= sort E by weight
//	cout<<"About to sort"<<endl;
	mGraph->sortEdges();
//	cout<<"done sorting"<<endl;
	
	EdgeList edges = mGraph->getEdges();
	
    //	  for u is an element of V
    //	      MAKESET(u)
    //	  rof
	
	// [ built into Vertex Class ]
	
    //	  for (u, v) is an element of E (in increasing order) do
    //	      if FIND(u) ≠ FIND(v) do
    //	          X = X union {(u,v)}
    //	          UNION(u,v)
    //    rof
    //	int size  = edges.size();
	for (EdgeList::iterator i = edges.begin(); i != edges.end(); ++i) {	
		Vertex* u = i->getStart();
		Vertex* v = i->getEnd();
		if ( find(u) != find(v) ) {
			tree->push_back(*i);		// add edge to minimum spanning tree
			setsUnion(u, v);			// merge sets
		}
	}

    //    return(X)
	return tree;
}


// make sure rand gen is good.  Do not use rand()

//
//  random seed according to time
//
void randomizer()
{
	time_t      timeval;
	
	time(&timeval);
	srand(timeval);
}
//
//  random generator
//
int randomInt(int min, int max) {
	static bool seed = false;
	if (! seed) {
            randomizer();
            seed = true;
	}
	
	int range = (max + 1) - min;
	return (int) (( rand() / (double) RAND_MAX ) * range + min);
}
// yield a random real number from 0 to 1
double randomDble() {
	static bool seed = false;
	if (! seed) {
            randomizer();
            seed = true;
	}
	
	double mReturn = 1.0;	// should not return 1.0
	do {
		mReturn = rand() / (double) RAND_MAX;
	} while (mReturn == 1.0);
	
	return mReturn;
}

void wait ( int seconds ) {
	clock_t endwait;
	endwait = clock() + seconds * CLK_TCK ;
	while (clock() < endwait) {}
}


// get edge by finding distance between pts

int main(int ac, char *av[]) {
	
	// defaults
	int randmst		= 0;	// random seed???
	int temp		= 0;	// unused.
	
//	int numpoints	= 2048;	// n
	int numtrials	= 10;	// runs done
	int dimension	= 2;	// mode  2 = square, 0 for random weights
	
	for (int numpoints = 16; numpoints <= 131072; numpoints *= 2) {
		float averageWeight = 0;
		for (int i = 0; i < numtrials; i++) {
//			cout << "trial #" << i+1 << endl;
			// create graph inside a unit square
			Graph mGraph = Graph();
	//		cout<<"about to start generating vertices"<<endl;
			mGraph.generateVertices(numpoints);
	//		cout<<"done generating vertices, about to connect"<<endl;
			mGraph.connectVertices(true);
	//		cout<<"done connecting vertices, about to compute min spanning tree"<<endl;
			// mGraph.printVertices();
			// mGraph.printEdges();
			
			EdgeList* minSpanTree = kruskal(&mGraph);
			float weight = 0;
	//		cout << "\nMIN SPAN TREE of: "<< minSpanTree->size() << " edges" << endl;
			for (EdgeList::iterator i = minSpanTree->begin(); i != minSpanTree->end(); ++i) {
				// i->pprint();
				weight += i->getEdgeWeight();
			}
			cout << "Tree [true] has weight of: " << weight << endl;
			averageWeight += weight;
			delete minSpanTree;
		}
		averageWeight /= numtrials;
		cout<<"Average weight of tree for n = "<<numpoints<<" averaged over "<<numtrials<<" tries: "<<averageWeight<<endl;
		
	
		averageWeight = 0;
		for (int i = 0; i < numtrials; i++) {
//			cout << "trial #" << i+1 << endl;
			// create graph inside a unit square
			Graph mGraph = Graph();
	
	
	//		cout<<"about to start generating vertices"<<endl;
			mGraph.generateVertices(numpoints);
	//		cout<<"done generating vertices, about to connect"<<endl;
			mGraph.connectVertices(false);
	//		cout<<"done connecting vertices, about to compute min spanning tree"<<endl;
			// mGraph.printVertices();
			// mGraph.printEdges();
			
				
			EdgeList* minSpanTree = kruskal(&mGraph);
			float weight = 0;
//			cout << "\nMIN SPAN TREE of: "<< minSpanTree->size() << " edges" << endl;
			for (EdgeList::iterator i = minSpanTree->begin(); i != minSpanTree->end(); ++i) {
	//			i->pprint();
				weight += i->getEdgeWeight();
			}
			cout << "Tree [false] has weight of: " << weight << endl;
			averageWeight += weight;
			delete minSpanTree;
		}
		averageWeight /= numtrials;
		cout<<"Average weight of tree for n = "<<numpoints<<" averaged over "<<numtrials<<" tries: "<<averageWeight<<endl;
	}
	
	
	/*
    if (ac != 5) {
	} else {
		randmst		= av[0];
		temp		= av[1];
		numpoints	= av[2];
		numtrials	= av[3];
		dimension	= av[4];
	}
	*/
	
	
    return 0;
}





	/*		// build simple graph from notes
	Graph mGraph = Graph();
	mGraph.addVertex(1, 1);
	mGraph.addVertex(2, 1);
	mGraph.addVertex(3, 1);
	mGraph.addVertex(1, 2);
	mGraph.addVertex(2, 2);
	mGraph.addVertex(3, 2);
	mGraph.addVertex(1, 3);
	mGraph.addVertex(2, 3);
	mGraph.addVertex(3, 3);
	mGraph.createEdge(0, 1, 1);
	mGraph.createEdge(1, 5, 2);
	mGraph.createEdge(3, 4, 4);
	mGraph.createEdge(4, 1, 5);
	mGraph.createEdge(6, 3, 7);
	mGraph.createEdge(7, 6, 8);
	mGraph.createEdge(0, 3, 3);
	mGraph.createEdge(3, 2, 6);
	mGraph.createEdge(1, 5, 4);
	mGraph.createEdge(4, 5, 7);
	mGraph.createEdge(2, 2, 5);
	mGraph.createEdge(5, 7, 8);
	*/
    