//
//
//  Matrix.cpp
//
//      AUTHOR: Russell Lowke
//
//		  Date: 4/15/2004
//

#include "Matrix.h"
#include <iostream>

using namespace std;

//
//	constructor:	create a matrix of specified width and height from data[]
//
Matrix::Matrix (int width, int height, int data[]): _Data(width), _Width(width), _Height(height) {

    for (int x = 0; x < width; ++x) {
        IntList col(height);
        for (int y = 0; y < height; ++y)
			col[y] = data[ width * y + x ];
        _Data[x] = col;
    }
}

//
//	constructor:	create matrix from IntLList
//
Matrix::Matrix (IntLList data) {
	_Width  = data.size();
    _Height = data[0].size();
    _Data   = data;
}

/*
//
//	constructor:	create matrix from four small matrices
//
Matrix::Matrix(Matrix a, Matrix b, Matrix c, Matrix d, int n): _Data(n) {

	_Width   = n;				// x
    _Height  = n;				// y
	IntLList _Data(_Width);
	
    int size = a.getWidth();    // size of small matrices
    int val;					// misc. holder
	
    // add elements
    for (int x = 0; x < _Width; ++x) {
        IntList col(_Height);					// make new column
        
        for (int y = 0; y < _Height; ++y ) {
            
            if (y <  size && x <  size)
                val = a._Data[x][y];
            else if (y <  size && x >= size)
                val = b._Data[x - size][y];
            else if (y >= size && x < size)
                val = c._Data[x][y - size];
            else if (y >= size && x >= size)
                val = d._Data[x - size][y - size];
            
            col[y] =val;
        }
        _Data[x] = col;		// add column to IntLList
    }
}
*/

//
//	constructor:	create matrix from four small matrices
//
Matrix::Matrix(Matrix a, Matrix b, Matrix c, Matrix d, int n): _Data(n) {

	_Width   = n;				// x
    _Height  = n;				// y
        
    // add elements
    Matrix* top;
    Matrix* bottom;
    for (int x = 0; x < n; ++x) {
        IntList col(n);					// make new column

        if (x == 0){
            top = &a;
            bottom = &c;
        } else if (x == _Width / 2) {
            top = &b;
            bottom = &d;
        }

        for (int y = 0; y < _Height; ++y ) {
            if ( x < _Width / 2){
                if ( y < _Width / 2){
                    col[y] = top->_Data[x][y];
                } else {
                    col[y] = bottom->_Data[x][y - n / 2];
                }
            } else {
                if ( y < _Width / 2){
                    col[y] = top->_Data[x - n / 2][y];
                } else {
                    col[y] = bottom->_Data[x - n / 2][y - n / 2];
                }
            }
        }
        _Data[x] = col;					// add column to IntLList
    }
	//cout<<"Done recombining"<<endl;
}


//
// add matrices
//
Matrix Matrix::operator+ ( const Matrix& other) const {

    IntLList mData(_Width);				// Data for resultant Matrix
    
    // ensure matrices are compatable
    if (_Width == other._Width && _Height == other._Height) {
        
        // add elements
        for (int x = 0; x < _Width; ++x) {
            IntList col(_Height);		// make new column
            for (int y = 0; y < _Height; ++y )
				col[y] = _Data[x][y] + other._Data[x][y];
			mData[x] = col;				// add column to IntLList
        }
    
    } else {
        // Matrices not compatable.
        cout << "Can't add these matrices" << endl;
    }
    
    return Matrix(mData);
}

//
// subtract matrices
//
Matrix Matrix::operator- ( const Matrix& other) const {

    IntLList mData(_Width);					// Data for resultant Matrix
    
    // ensure matrices are compatable
    if (_Width == other._Width && _Height == other._Height) {
        
        // add elements
        for (int x = 0; x < _Width; ++x) {
            IntList col(_Height);			// make new column
            for (int y = 0; y < _Height; ++y )
                col[y] = _Data[x][y] - other._Data[x][y];
            mData[x] = col;			// add column to IntLList
        }
    
    } else {
        // Matrices not compatable.
        cout << "Can't subtract these matrices" << endl;
    }
    
    return Matrix(mData);
}


//
// multiply matrices
//
Matrix Matrix::operator* ( const Matrix& other) const {
    
    // Matrix will have m's height and n's width
    int height = _Height;
    int width  = other._Width;
    
	IntLList mData(width);					// Data for resultant Matrix
	
    for (int x = 0; x < width; ++x) {
        IntList col(height);				// make new column
        for (int y = 0; y < height; ++y ) {
            int val = 0;
            for (int i = 0; i < height; ++i )
                val += (_Data[i][y] * other._Data[x][i] );
            col[y] = val;					// add column to Matrix
        }
        mData[x] = col;
    }
    
    return Matrix(mData);
}


//
//	print Matrix
//
void Matrix::print() const {
    
    for (int y = 0; y < _Height; ++y) {
        cout << "[ ";
        
        for (int x = 0; x < _Width; ++x) {
            cout << _Data[x][y] << " ";
        }
        
        cout << "]" << endl;
    }
    
    cout << endl;
}
    
	
//
//	print Matrix
//
void Matrix::printDiag() const {
    
	cout << "[ ";
	
    for (int y = 0; y < _Height; ++y)
        for (int x = 0; x < _Width; ++x)
			if (x == y)
				cout << _Data[x][y] << " ";

    cout << "]" << endl;
	
    cout << endl;
}


//
//  Partition Matrix into quarters
//			Note: Matrix assumed to be square and even [div. by 2] in size.
MatrixList Matrix::makeQuarters() {
    
    MatrixList mReturn(4);
    
    int n = _Width / 2;					// small matrix size
    
    int data[4][n * n];					// data for each new 1/4	
    int counter[] = {-1, -1, -1, -1};
    int quarter;						// quarter being used
    
    for (int y = 0; y < (_Width); ++y ) {
        for (int x = 0; x < (_Width); ++x) {

            if (x <= n-1 && y <= n-1)
                quarter = 0;			// top left
            else if (x >  n-1 && y <= n-1)
                quarter = 1;			// top right
            else if (x <= n-1 && y >  n-1)
                quarter = 2;			// bottom left
            else if (x >  n-1 && y >  n-1)
                quarter = 3;			// bottom right
            
            data[quarter][++counter[quarter]] = _Data[x][y];
        }
    }
    
    for (int i = 0; i < 4; ++i)
        mReturn[i] = Matrix(n, n, data[i]);
        
    return mReturn;
}


void Matrix::addPadding() {
    for (int i = 0; i < _Width; ++i) {
        _Data[i].push_back(0);			// add 0 to each column
    }
    ++_Height;
    
    IntList col;						// create padded column
    for (int i = 0; i < _Height; ++i)
        col.push_back(0);
        
    _Data.push_back( col );				// add new column
    ++_Width;
}


void Matrix::removePadding() {
    _Data.pop_back();					// remove last colum of 0s
    --_Width;

    for (int i = 0; i < _Width; ++i) {
        _Data[i].pop_back();			// remove 0 from each column
    }
    --_Height;
}


//
//  accessors and mutators
//
void	Matrix::setAt(int x, int y, int val)	{ _Data[x][y] = val;  }
int		Matrix::getAt(int x, int y)				{ return _Data[x][y]; }
int		Matrix::getWidth()						{ return _Width;      }
int		Matrix::getHeight()						{ return _Height;     }
