//
//
//  strassen.cpp
//
//      AUTHOR: Russell Lowke
//
//          Date: 4/14/2004
//


#include <iostream>
#include <cstring>
#include <string>
#include <fstream>

using std::string;

#include "Matrix.h"

void wait ( int seconds ) {
	clock_t endwait;
	endwait = clock() + seconds * CLK_TCK ;
	while (clock() < endwait) {}
}


Matrix mms(Matrix m, Matrix n, int d) {
        
    Matrix mReturn;
    
    if (d == 1)
        mReturn = m * n;					// resultant 1 x 1 matrix
    else {
    
        bool paddingAdded = false;
        if (d % 2 != 0 ) {
            m.addPadding();
            n.addPadding();
            ++d;
            paddingAdded = true;
        }
    
        // separate each matrix into 4 smaller matrices
        // 	       [ A  B ]            [ E  F ]
        //		   [ C  D ]            [ G  H ]
        
        // 	       [ mGrp[0]  mGrp[1] ][ nGrp[0]  nGrp[1] ]
        //		   [ mGrp[2]  mGrp[3] ][ nGrp[2]  nGrp[3] ]
        MatrixList mGrp = m.makeQuarters();
        MatrixList nGrp = n.makeQuarters();
        int sd = mGrp[0].getWidth();
            
        // compute P1 thru 7 with recursive calls to mms
        
        // p1 = A(F - H)
        Matrix p1 = mms(mGrp[0], (nGrp[1] - nGrp[3]), sd);
        // p2 = (A + B)H			
        Matrix p2 = mms((mGrp[0] + mGrp[1]), nGrp[3], sd);
        // p3 = (C + D)E			
        Matrix p3 = mms((mGrp[2] + mGrp[3]), nGrp[0], sd);
        // p4 = D(G - E)			
        Matrix p4 = mms(mGrp[3], (nGrp[2] - nGrp[0]), sd);
        // p5 = (A + D)(E + H)		
        Matrix p5 = mms((mGrp[0] + mGrp[3]), (nGrp[0] + nGrp[3]), sd);
        // p6 = (B - D)(G + H)
        Matrix p6 = mms((mGrp[1] - mGrp[3]), (nGrp[2] + nGrp[3]), sd);
        // p7 = (A - C)(E + F)
        Matrix p7 = mms((mGrp[0] - mGrp[2]), (nGrp[0] + nGrp[1]), sd);
        
        /*
        Matrix p1 = mGrp[0] * (nGrp[1] - nGrp[3]);				// p1 = A(F - H)
        Matrix p2 = (mGrp[0] + mGrp[1]) * nGrp[3];				// p2 = (A + B)H
        Matrix p3 = (mGrp[2] + mGrp[3]) * nGrp[0];				// p3 = (C + D)E
        Matrix p4 = mGrp[3] * (nGrp[2] - nGrp[0]);				// p4 = D(G - E)
        Matrix p5 = (mGrp[0] + mGrp[3]) * (nGrp[0] + nGrp[3]);	// p5 = (A + D)(E + H)
        Matrix p6 = (mGrp[1] - mGrp[3]) * (nGrp[2] + nGrp[3]);	// p6 = (B - D)(G + H)
        Matrix p7 = (mGrp[0] - mGrp[2]) * (nGrp[0] + nGrp[1]);	// p7 = (A - C)(E + F)
        */
        
        
        // compute [ AE + BG   AF + BH ]
        //		   [ CE + DG   CF + DH ]
        
        // AE + BG  = p5 + p4 - p2 + p6
        // AF + BH  = p1 + p2
        // CE + DG  = p3 + p4
        // CF + DH  = p5 + p1 - p3 - p7
        
        // fill resultant matrix and return
        mReturn = Matrix( p5 + p4 - p2 + p6,
                          p1 + p2, 
                          p3 + p4, 
                          p5 + p1 - p3 - p7,
                          d );
        
        if (paddingAdded)
            mReturn.removePadding();
    }
    
    return mReturn;
}
//
// multiply matrices strassen   m x n
//
Matrix mms(Matrix m, Matrix n) {

    // get dimension
    int d = m.getWidth();
    
    if (d != m.getHeight() ||
        d != n.getWidth()  || 
        d != n.getHeight()) {
        
        cout << "Cannot Strassen multiply matrices of different dimensions." << endl;
        return Matrix();
        
    } else
        return mms(m, n, d);
}

void pprintArray(int* array, int d) {

    for (int i = 0; i < d*d; ++i) {
        cout << array[i] << ", ";
    }
    cout << endl;
}

Matrix* readFile (string filename, int d) {
    
    //MatrixList mReturn(2);

//    Matrix matrixA, matrixB;
    int dataA[d*d];
    int dataB[d*d];
    
    cout << "Creating document from " + filename << endl;
    
    fstream inputFile(filename.c_str(), ios::in);
    
    if (!inputFile)
        cerr << "ERR: File not found" << endl;
    else {
        int val, nItems = d * 2;
        //dataA.size() < d
        
        for (int x = 0; x < nItems; ++x) {
            for(int y = 0; y < d; ++y) {
                inputFile>>val;
                
                if (x < d) {
                    //cout << "A (" << x << ", " << y <<") = "<< val<< endl;
                    dataA[x+y*d] = val;			// add to first Matrix
                } else {
                    //cout << "B (" << (x-d) << ", " << y <<") = "<< val<< endl;
                    dataB[(x-d)+y*d] = val;			// add to second Matrix
                }
            }
        }
    }
    
    typedef Matrix* MatrixPtr;
    
    cout<<"Data for B on the outside is:"<<endl;
    pprintArray(dataB, d);
    MatrixPtr matrixB = new Matrix(d, d, dataB);
    matrixB->print();
    
    cout<<"Data for A on the outside is:"<<endl;
    pprintArray(dataA, d);
    MatrixPtr matrixA = new Matrix(d, d, dataA);
    matrixA->print();
    

    
    matrixA->print();
    matrixB->print();
    //cout << "rtest" << endl;


    
    MatrixPtr mReturn;
    /*
    mReturn[0] = matrixA;
    mReturn[1] = matrixB;
    */
/*
    mReturn[0]=(  );
    mReturn[1]=(  );
*/    
    return mReturn;
    
}



int main(int argc, char* argv[]) {
    
    // strassen 0 dimension textfile
    
    int d;
    string filename;

    if (argc == 4) {
        d        = atoi(argv[2]);
        filename = argv[3];
    } else {
        cerr << "No arguments: using defaults" << endl;
        d = 3;
        filename = "Matrix.txt";
    }
    
    // read textfile.
    Matrix* data = readFile(filename, d);
    Matrix matrixA = data[0];
    Matrix matrixB = data[1];
    
    matrixA.print();
    cout << "x" << endl;
    matrixB.print();
    
	cout << "TIME Start:" << clock() << endl;
    cout << "=" << endl;
    Matrix matrixC = matrixA * matrixB;
    matrixC.print();
	
	cout << "TIME prestrassen:" << clock() << endl;
	
    cout << "Strassen" << endl;
    Matrix matrixD = mms(matrixA, matrixB);
    matrixD.print();
    
	cout << "TIME End:" << clock() << endl;
}