//
//
//  strassen.cpp
//
//      AUTHOR: Russell Lowke
//
//          Date: 4/14/2004
//


#include <iostream>
#include <cstring>
#include <string>

using std::string;

#include "Matrix.h"

void wait ( int seconds ) {
	clock_t endwait;
	endwait = clock() + seconds * CLK_TCK ;
	while (clock() < endwait) {}
}



Matrix mms(Matrix m, Matrix n, int d) {
        
    Matrix mReturn;
    
    if (d == 1)
        mReturn = m * n;					// resultant 1 x 1 matrix
    else {
    
        bool paddingAdded = false;
        if (d % 2 != 0 ) {
            m.addPadding();
            n.addPadding();
            ++d;
            paddingAdded = true;
        }
    
        // separate each matrix into 4 smaller matrices
        // 	       [ A  B ]            [ E  F ]
        //		   [ C  D ]            [ G  H ]
        
        // 	       [ mGrp[0]  mGrp[1] ][ nGrp[0]  nGrp[1] ]
        //		   [ mGrp[2]  mGrp[3] ][ nGrp[2]  nGrp[3] ]
        MatrixList mGrp = m.makeQuarters();
        MatrixList nGrp = n.makeQuarters();
        int sd = mGrp[0].getWidth();
            
        // compute P1 thru 7 with recursive calls to mms
        
        // p1 = A(F - H)
        Matrix p1 = mms(mGrp[0], (nGrp[1] - nGrp[3]), sd);
        // p2 = (A + B)H			
        Matrix p2 = mms((mGrp[0] + mGrp[1]), nGrp[3], sd);
        // p3 = (C + D)E			
        Matrix p3 = mms((mGrp[2] + mGrp[3]), nGrp[0], sd);
        // p4 = D(G - E)			
        Matrix p4 = mms(mGrp[3], (nGrp[2] - nGrp[0]), sd);
        // p5 = (A + D)(E + H)		
        Matrix p5 = mms((mGrp[0] + mGrp[3]), (nGrp[0] + nGrp[3]), sd);
        // p6 = (B - D)(G + H)
        Matrix p6 = mms((mGrp[1] - mGrp[3]), (nGrp[2] + nGrp[3]), sd);
        // p7 = (A - C)(E + F)
        Matrix p7 = mms((mGrp[0] - mGrp[2]), (nGrp[0] + nGrp[1]), sd);
        
        /*
        Matrix p1 = mGrp[0] * (nGrp[1] - nGrp[3]);				// p1 = A(F - H)
        Matrix p2 = (mGrp[0] + mGrp[1]) * nGrp[3];				// p2 = (A + B)H
        Matrix p3 = (mGrp[2] + mGrp[3]) * nGrp[0];				// p3 = (C + D)E
        Matrix p4 = mGrp[3] * (nGrp[2] - nGrp[0]);				// p4 = D(G - E)
        Matrix p5 = (mGrp[0] + mGrp[3]) * (nGrp[0] + nGrp[3]);	// p5 = (A + D)(E + H)
        Matrix p6 = (mGrp[1] - mGrp[3]) * (nGrp[2] + nGrp[3]);	// p6 = (B - D)(G + H)
        Matrix p7 = (mGrp[0] - mGrp[2]) * (nGrp[0] + nGrp[1]);	// p7 = (A - C)(E + F)
        */
        
        
        // compute [ AE + BG   AF + BH ]
        //		   [ CE + DG   CF + DH ]
        
        // AE + BG  = p5 + p4 - p2 + p6
        // AF + BH  = p1 + p2
        // CE + DG  = p3 + p4
        // CF + DH  = p5 + p1 - p3 - p7
        
        
        
        // fill resultant matrix and return
        mReturn = Matrix( p5 + p4 - p2 + p6,
                          p1 + p2, 
                          p3 + p4, 
                          p5 + p1 - p3 - p7,
                          d );
        
        if (paddingAdded)
            mReturn.removePadding();
    }
    
    return mReturn;
}
//
// multiply matrices strassen   m x n
//
Matrix mms(Matrix m, Matrix n) {

    // get dimension
    int d = m.getWidth();
    
    if (d != m.getHeight() ||
        d != n.getWidth()  || 
        d != n.getHeight()) {
        
        cout << "Cannot Strassen multiply matrices of different dimensions." << endl;
        return Matrix();
        
    } else
        return mms(m, n, d);
}


int main(int argc, char* argv[]) {
    
    // strassen 0 dimension textfile
    
    int d;
    string filename;

    if (argc == 3) {
        d        = atoi(argv[1]);
        filename = argv[2];
    } else {
        cerr << "No arguments: using defaults" << endl;
        d = 12;
        filename = "Matrix.txt";
    }
    
    
    /* read textfile.
    
    matrixA = readFile(filename, d);
    matrixB = readFile(filename, d);
    */
    
    
	int dataA[] = {1, 2, 3, 4, 5, 6, 7, 8, 9};
    Matrix matrixA = Matrix(3, 3, dataA );
    Matrix matrixB = matrixA;
    
    matrixA.print();
    cout << "x" << endl;
    matrixB.print();
    
    cout << "=" << endl;
    Matrix matrixC = matrixA * matrixB;
    matrixC.print();

    cout << "Strassen" << endl;
    Matrix matrixD = mms(matrixA, matrixB);
    matrixD.print();
}