//
//
//  numberPartition
//
//      AUTHOR: Russell Lowke
//
//		  Date: 5/15/2004
//


#include <iostream>
using std::cout;
using std::endl;

#include <fstream>
using std::fstream;
using std::ios;

#include "heuristic.h"

void printAnswer(int* answer) {
    cout << "{";
    for (int i = 0; i < 100; ++i) {
        cout << answer[i] << " ";
    }
    cout << "}" << endl;
}

void printAnswerToFile(int* answer, fstream& log) {
    log << "{";
    for (int i = 0; i < 100; ++i) {
        log << answer[i] << " ";
    }
    log << "}" << endl;
}

void printInstanceToFile(bigint* instance, fstream& log) {
    log << "{";
    for (int i = 0; i < 100; ++i) {
        log << instance[i] << " ";
    }
    log << "}" << endl;
}


int main (int argc, char * const argv[]) {
	fstream logFile("NumberPartition.log", ios::out);
	randomizer();
	for(int i = 0; i < 50; ++i){
	
		cout << i+1 << ")" << endl;
		logFile << i+1 << ")" << endl;
		
		bigint instance[100];
		generateRandomInstance(instance);
		cout << "Random instance generated\n";
		logFile << "instance: "; 
		printInstanceToFile( instance, logFile );

		int* answer;
		int* answerFromP;

		cout << "Answer from KK heuristic is\n";
		cout << KKsolution(instance) << endl;

		logFile << "Answer from KK heuristic is\n";
		logFile << KKsolution(instance) << endl;
		
		cout << "About to start computing hill climbing answers\n";

		cout << "hillClimbing answer using S is:\n";
		answer = hillClimbing(instance);
		printAnswer(answer);
		logFile << "hillClimbing answer using S is:\n";
		printAnswerToFile(answer, logFile);
		
		cout << "Residue is: " << calculateResidue(instance, answer) << endl;
		logFile << "Residue is: " << calculateResidue(instance, answer) << endl;
		delete[] answer;

		cout << "hillClimbing answer using P is:\n";
		answerFromP = hillClimbingWithP(instance);
		printAnswer(answerFromP);
		cout << "Residue is: " << calculateResidueWithP(instance, answerFromP) << endl;
		logFile << "hillClimbing answer using P is:\n";
		printAnswerToFile(answerFromP, logFile);
		logFile << "Residue is: " << calculateResidueWithP(instance, answerFromP) << endl;
		delete[] answerFromP;
		
				
		cout << "About to start computing random answers\n";

		cout << "randomSolution answer using S is:\n";
		answer = randomSolution(instance);
		printAnswer(answer);
		logFile << "randomSolution answer using S is:\n";
		printAnswerToFile(answer, logFile);
		cout << "Residue is: " << calculateResidue(instance, answer) << endl;
		logFile << "Residue is: " << calculateResidue(instance, answer) << endl;
		delete[] answer;

		cout << "randomSolution answer using P is:\n";
		answerFromP = randomSolutionWithP(instance);
		printAnswer(answerFromP);
		logFile << "randomSolution answer using P is:\n";
		printAnswerToFile(answerFromP, logFile);
		
		cout << "Residue is: " << calculateResidueWithP(instance, answerFromP) << endl;
		logFile << "Residue is: " << calculateResidueWithP(instance, answerFromP) << endl;
		delete[] answerFromP;

		cout << "About to start computing simulatedAnnealing answers\n";

		cout << "simulatedAnnealing answer using S is:\n";
		answer = simulatedAnnealing(instance);
		printAnswer(answer);
		cout << "Residue is: " << calculateResidue(instance, answer) << endl;
		
		logFile << "simulatedAnnealing answer using S is:\n";
		printAnswerToFile(answer, logFile);
		logFile << "Residue is: " << calculateResidue(instance, answer) << endl;
		delete[] answer;

		answerFromP = simulatedAnnealingWithP(instance);
		cout << "simulatedAnnealing answer using P is:\n";
		printAnswer(answerFromP);
		cout << "Residue is: " << calculateResidueWithP(instance, answerFromP) << endl;
		
		logFile << "simulatedAnnealing answer using P is:\n";
		printAnswerToFile(answerFromP, logFile);
		logFile << "Residue is: " << calculateResidueWithP(instance, answerFromP) << endl;
		delete[] answerFromP;
		
		cout << endl;
		logFile << endl;
	}	
	
    return 0;
}