///////////
//
//  Graph.hpp
//              header for Graph.cpp package
//
//      AUTHOR: Russell James Lowke  (lowke@fas.harvard.edu)
//
//		  Date: 3/13/2004
//

#ifndef _Graph_
#define _Graph_

#include <vector>
using std::vector;

typedef vector< class Edge>	EdgeList;
typedef vector<class Vertex*> VertexList;

class Graph {
	  
	public:
        
	  // methods
	  Graph();	// constructor
	  Graph(VertexList mVertices, EdgeList mEdges);		// constructor
      ~Graph();											// destructor
	  void			addVertex(double mX, double mY);	// add a vertex to Graph at x,y
	  void			addEdge(Edge mEdge);				// add an Edge to this Graph
	  void			createEdge(int mStart, double mWeight, int mEnd);
	  void			createEdge(Vertex* mStart, double mWeight, Vertex* mEnd);
	  void			printVertices();			
	  void			printEdges();
	  void			sortEdges();						// sorts edges from smallest to highest
	  void			generateVertices(int n);			// generate n verticies
	  void			connectVertices(bool distWeight);	// connects every vertex with every other
	  VertexList	getVertices() const;
	  EdgeList		getEdges() const;
	  
	private:

	  VertexList	vertices;							// list of Vertices in Graph
	  EdgeList		edges;								// list of Edges in Graph
 
	  void			edgeQuicksort(int low, int high);	// used by sortEdges()
	  int			edgeSplit(int low, int high);		// used by sortEdges()
	  void			addVertex(Vertex* mVertex);			// internal add a vertex
	  double		distance(Vertex* thisVertex, Vertex* otherVertex);
};

#endif /* _Graph_ */
