//
//
//  randmst.cpp
//
//      AUTHOR: Russell Lowke
//
//		  Date: 3/13/2004
//


#include "edge.hpp"
#include "graph.hpp"
#include "vertex.hpp"

#include <cstring>
#include <iostream>
#include <ctime>
#include <vector>

using namespace std;

// return the name of the set containing the element x
Vertex* find(Vertex* mX) {
	
	// if x ‚â† p(x) then
    //	p(x) := FIND(p(x))
    //	return (p(x))
	if (mX != mX->getParent())
		mX->setMySet(find(mX->getParent()));		// path compression

	return mX->getParent();
}

Vertex* link(Vertex* mX, Vertex* mY) {

    // if rank(x) > rank(y) 
	if (mX->getRank() > mY->getRank()) {
		// then switch x and y
		Vertex* mTemp = mY;
		mY = mX;
		mX = mTemp;
	}
	
    // if rank(x) = rank(y) then rank(y) := rank(y) + 1
	if (mX->getRank() == mY->getRank())
		mY->setRank(mY->getRank() + 1);
	
    // p(x) := y
	mX->setMySet(mY);
	
    // return(y)
	return mY;
}


// replace two sets containing x and y by their union
void setsUnion(Vertex* mX, Vertex* mY) {
	//     LINK(FIND(x), FIND(y))
	link(find(mX), find(mY));
}


// Function Kruskal(graph G(V,E))
//    set X
//	  X = {}
//	  E:= sort E by weight
//	  for u is an element of V
//	      MAKESET(u)
//	  rof
//	  for (u, v) is an element of E (in increasing order) do
//	      if FIND(u) ‚â† FIND(v) do
//	          X = X union {(u,v)}
//	          UNION(u,v)
//    rof
//    return(X)
// end Kruskal
//
EdgeList* kruskal(Graph* mGraph) {
    
    //    set X
    //	  X = {}
	EdgeList* tree = new EdgeList;

    //	  E:= sort E by weight
	mGraph->sortEdges();
	EdgeList edges = mGraph->getEdges();
	
    //	  for u is an element of V
    //	      MAKESET(u)
    //	  rof
	
	// [ built into Vertex Class ]
	
    //	  for (u, v) is an element of E (in increasing order) do
    //	      if FIND(u) ‚â† FIND(v) do
    //	          X = X union {(u,v)}
    //	          UNION(u,v)
    //    rof
    //	int size  = edges.size();
	for (EdgeList::iterator i = edges.begin(); i != edges.end(); ++i) {	
		Vertex* u = i->getStart();
		Vertex* v = i->getEnd();
		if ( find(u) != find(v) ) {
			tree->push_back(*i);		// add edge to minimum spanning tree
			setsUnion(u, v);			// merge sets
		}
	}

    //    return(X)
	return tree;
}


//
//  random seed according to time
//
void randomizer()
{
	time_t      timeval;
	
	time(&timeval);
	srand(timeval);
}
//
//  random generator
//
int randomInt(int min, int max) {
	static bool seed = false;
	if (! seed) {
            randomizer();
            seed = true;
	}
	
	int range = (max + 1) - min;
	return (int) (( rand() / (double) RAND_MAX ) * range + min);
}
// yield a random real number from 0 to 1
double randomDble() {
	static bool seed = false;
	if (! seed) {
            randomizer();
            seed = true;
	}
	
	double mReturn = 1.0;	// should not return 1.0
	do {
		mReturn = rand() / (double) RAND_MAX;
	} while (mReturn == 1.0);
	
	return mReturn;
}

// [ not used ]
void wait ( int seconds ) {
	clock_t endwait;
	endwait = clock() + seconds * CLK_TCK ;
	while (clock() < endwait) {}
}

int main(int ac, char *av[]) {
	
	// randmst 0 numpoints numtrials dimension
	
	// defaults
	int temp		= 0;	// unused.
	int numpoints	= 12;	// n
	int numtrials	= 5;	// runs done
	int dimension	= 2;	// mode  2 = square, 0 for random weights
	

    if (ac == 5) {
		temp		= atoi(av[1]);
		numpoints	= atoi(av[2]);
		numtrials	= atoi(av[3]);
		dimension	= atoi(av[4]);
	}

	cout << "numpoints = " << numpoints << endl
		 << "numtrials = " << numtrials << endl
		 << "dimension = " << dimension;
		 
	// set flag for using distance between points
	bool useDistance = false;
	if (dimension == 2)
		useDistance = true;
	
	Graph mGraph;
	EdgeList* minSpanTree;
	
	for (int counter = 0; counter < numtrials; ++counter) {
		
		cout << endl 
		     << endl << "trial #" << counter << "  n = " << numpoints << endl;
		
		// create graph inside a unit square
		mGraph = Graph();
		mGraph.generateVertices(numpoints);
		mGraph.connectVertices(useDistance);
		
		minSpanTree = kruskal(&mGraph);
		float weight = 0;
		cout << endl 
			 << "Minimum spanning tree of: "
			 << minSpanTree->size() << " edges" << endl;
		
		double largestEdge = 0;
		for (EdgeList::iterator i = minSpanTree->begin(); i != minSpanTree->end(); ++i) {
			if (i->getEdgeWeight() > largestEdge)
				largestEdge = i->getEdgeWeight();
			weight += i->getEdgeWeight();
		}
		
		cout << "Largest edge in minSpanTree: " << largestEdge << endl;
		cout << "Tree has weight of: " << weight << endl;
		delete minSpanTree;
		
	}
	
    return 0;
}


	/*		// build simple graph from notes
	Graph mGraph = Graph();
	mGraph.addVertex(1, 1);
	mGraph.addVertex(2, 1);
	mGraph.addVertex(3, 1);
	mGraph.addVertex(1, 2);
	mGraph.addVertex(2, 2);
	mGraph.addVertex(3, 2);
	mGraph.addVertex(1, 3);
	mGraph.addVertex(2, 3);
	mGraph.addVertex(3, 3);
	mGraph.createEdge(0, 1, 1);
	mGraph.createEdge(1, 5, 2);
	mGraph.createEdge(3, 4, 4);
	mGraph.createEdge(4, 1, 5);
	mGraph.createEdge(6, 3, 7);
	mGraph.createEdge(7, 6, 8);
	mGraph.createEdge(0, 3, 3);
	mGraph.createEdge(3, 2, 6);
	mGraph.createEdge(1, 5, 4);
	mGraph.createEdge(4, 5, 7);
	mGraph.createEdge(2, 2, 5);
	mGraph.createEdge(5, 7, 8);
	*/
    